@@ -8,7 +8,7 @@ package Module::Build;
 
 use strict;
 use vars qw($VERSION @ISA);
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -53,21 +53,30 @@ my %OSTYPES = qw(
 		 mpeix     MPEiX
 		);
 
-if (eval "use Module::Build::Platform::$^O") {
-  @ISA = ("Module::Build::Platform::$^O");
+use File::Spec;
+
+# We only use this once - don't waste a symbol table entry on it.
+# More importantly, don't make it an inheritable method.
+my $load = sub {
+  my $mod = shift;
+  #warn "Using $mod";
+  eval "use $mod";
+  die $@ if $@;
+  @ISA = ($mod);
+};
+
+if (grep {-e File::Spec->catfile($_, qw(Module Build Platform), $^O) . '.pm'} @INC) {
+  $load->("Module::Build::Platform::$^O");
 
 } elsif (exists $OSTYPES{$^O}) {
-  eval "use Module::Build::Platform::$OSTYPES{$^O}";
-  die $@ if $@;
-  @ISA = ("Module::Build::Platform::$OSTYPES{$^O}");
+  $load->("Module::Build::Platform::$OSTYPES{$^O}");
 
 } else {
   warn "Unknown OS type '$^O' - using default settings\n";
-  eval "use Module::Build::Platform::Default";
-  die $@ if $@;
-  @ISA = qw(Module::Build::Platform::Default);
+  $load->("Module::Build::Platform::Default");
 }
 
+sub os_type { $OSTYPES{$^O} }
 
 1;
 __END__
@@ -82,21 +91,13 @@ Module::Build - Build and install Perl modules
  Standard process for building & installing modules:
  
    perl Build.PL
-   ./Build             # this script created by 'perl Build.PL'
+   ./Build
    ./Build test
    ./Build install
- 
- Other actions:
- 
-   ./Build clean
-   ./Build realclean
-   ./Build fakeinstall
-   ./Build dist
-   ./Build help
 
 =head1 DESCRIPTION
 
-This is a very alpha version of a new module set I've been working on,
+This is a beta version of a new module set I've been working on,
 C<Module::Build>.  It is meant to be a replacement for
 C<ExtUtils::MakeMaker>.
 
@@ -104,7 +105,7 @@ To install C<Module::Build>, and any other module that uses
 C<Module::Build> for its installation process, do the following:
 
    perl Build.PL
-   ./Build
+   ./Build             # this script is created by 'perl Build.PL'
    ./Build test
    ./Build install
 
@@ -116,8 +117,8 @@ Other actions so far include:
    ./Build dist
    ./Build help
 
-It's like the C<MakeMaker> metaphor, except that C<Build> is a
-Perl script, not a Makefile.  State is stored in a directory called
+It's like the C<MakeMaker> metaphor, except that C<Build> is a short
+Perl script, not a long Makefile.  State is stored in a directory called
 C<_build/>.
 
 Any customization can be done simply by subclassing C<Module::Build> and
@@ -144,7 +145,7 @@ stabilizes.
 
 =over 4
 
-=item * Module::Build->new(...)
+=item * $m = Module::Build->new(...)
 
 Creates a new Module::Build object.  The C<module_name> argument is
 required, and should be a string like C<'Your::Module'>.  The
@@ -153,7 +154,18 @@ we'll look for the version string in the module specified by
 C<module_name>, parsing it out according to the same rules as
 C<ExtUtils::MakeMaker> and C<CPAN.pm>.
 
-=item * add_to_cleanup
+An optional C<c_source> argument specifies a directory which contains
+C source files that the rest of the build may depend on.  Any C<.c>
+files in the directory will be compiled to object files.  The
+directory will be added to the search path during the compilation and
+linking phases of any C or XS files.
+
+An optional C<autosplit> argument specifies a file which should be run
+through the C<Autosplit::autosplit()> function.  In general I don't
+consider this a great idea, and I may even go so far as to remove this
+feature later.  Let me know if I shouldn't.
+
+=item * $m->add_to_cleanup
 
 A C<Module::Build> method may call C<< $self->add_to_cleanup(@files) >>
 to tell C<Module::Build> that certain files should be removed when the
@@ -163,7 +175,7 @@ static lists can get difficult to manage.  I preferred to keep the
 responsibility for registering temporary files close to the code that
 creates them.
 
-=item * resume
+=item * Module::Build->resume
 
 You'll probably never call this method directly, it's only called from
 the auto-generated C<Build> script.  The C<new()> method is only
@@ -171,7 +183,7 @@ called once, when the user runs C<perl Build.PL>.  Thereafter, when
 the user runs C<Build test> or another action, the C<Module::Build>
 object is created using the C<resume()> method.
 
-=item * dispatch
+=item * $m->dispatch
 
 This method is also called from the auto-generated C<Build> script.
 It parses the command-line arguments into an action and an argument
@@ -181,6 +193,17 @@ C<ACTION_foo> method.  All arguments (including everything mentioned
 in L<ACTIONS> below) are contained in the C<< $self->{args} >> hash
 reference.
 
+=item * $m->os_type
+
+If you're subclassing Module::Build and some code needs to alter its
+behavior based on the current platform, you may only need to know
+whether you're running on Windows, Unix, MacOS, VMS, etc. and not the
+fine-grained value of Perl's C<$^O> variable.  The C<os_type()> method
+will return a string like C<Windows>, C<Unix>, C<MacOS>, C<VMS>, or
+whatever is appropriate.  If you're running on an unknown platform, it
+will return C<undef> - there shouldn't be many unknown platforms
+though.
+
 =back
 
 
@@ -239,6 +262,11 @@ with C<.t> in a C<t/> directory.
 If you want tests to be 'verbose', i.e. show details of test execution
 rather than just summary information, pass the argument C<verbose=1>.
 
+In addition, if a file called C<visual.pl> exists in the top-level
+directory, this file will be executed as a Perl script and its output
+will be shown to the user.  This is a good place to put speed tests or
+other tests that don't use the C<Test::Harness> format for output.
+
 =item * clean
 
 This action will clean up any files that the build process may have
@@ -280,7 +308,24 @@ actually run the C<install> action.
 
 This action is helpful for module authors who want to package up their
 module for distribution through a medium like CPAN.  It will create a
-tarball and compress it using GZIP compression.
+tarball of the files listed in MANIFEST and compress the tarball using
+GZIP compression.
+
+=item * manifest
+
+This is an action intended for use by module authors, not people
+installing modules.  It will bring the MANIFEST up to date with the
+files currently present in the distribution.  You may use a
+MANIFEST.SKIP file to exclude certain files or directories from
+inclusion in the MANIFEST.  MANIFEST.SKIP should contain a bunch of
+regular expressions, one per line.  If a file in the distribution
+directory matches any of the regular expressions, it won't be included
+in the MANIFEST.
+
+(Note to self: it would be nice to have a 'fake_manifest' action that
+would just go through the motions of adding to MANIFEST but not
+actually do anything.  Currently ExtUtils::Manifest doesn't support
+it, though.)
 
 =item * help
 
@@ -290,6 +335,31 @@ actions too.
 
 =back
 
+=head1 AUTOMATION
+
+One advantage of Module::Build is that since it's implemented as Perl
+methods, you can invoke these methods directly if you want to install
+a module non-interactively.  For instance, the following Perl script
+will invoke the entire build/install procedure:
+
+ my $m = new Module::Build (module_name => 'MyModule');
+ $m->dispatch('build');
+ $m->dispatch('test');
+ $m->dispatch('install');
+
+If any of these steps encounters an error, it will throw a fatal
+exception.
+
+You can also pass arguments as part of the build process:
+
+ my $m = new Module::Build (module_name => 'MyModule');
+ $m->dispatch('build');
+ $m->dispatch('test', verbose => 1);
+ $m->dispatch('install', sitelib => '/my/secret/place/');
+
+Building and installing modules in this way skips creating the
+C<Build> script.
+
 =head1 STRUCTURE
 
 Module::Build creates a class hierarchy conducive to customization.
@@ -384,6 +454,30 @@ one example, at L<http://www.dsmit.com/cons/> .
 
 Please contact me if you have any questions or ideas.
 
+=head1 TO DO
+
+Need to implement a prerequisite mechanism, similar to MakeMaker's
+C<PREREQ_PM> stuff.
+
+There will also be a subclassing mechanism that doesn't require as
+much module infrastructure to use.  Something like this:
+
+ use Module::Build subclass => <<'EOF';
+  sub ACTION_foo {
+    ... implement the 'foo' action ...
+  }
+ EOF
+
+The current method of relying on time stamps to determine whether a
+derived file is out of date isn't likely to scale well, since it
+requires tracing all dependencies backward, it runs into problems on
+NFS, and it's just generally flimsy.  It would be better to use an MD5
+signature or the like, if available.  See C<cons> for an example.
+
+The current dependency-checking for .xs files is prone to errors.  You
+can make 'widowed' files by doing C<Build>, C<perl Build.PL>, and then
+C<Build realclean>.  Should be easy to fix, but it's got me wondering
+whether the dynamic declaration of dependencies is a good idea.
 
 =head1 AUTHOR
 
